package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.EnumOperationType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;
import pwc.taxtech.atms.dto.vatdto.InputInvoiceDataSourceDto;
import pwc.taxtech.atms.vat.entity.InputInvoice;

import java.math.BigDecimal;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/// <summary>
/// 名称：本期/前期认证相符且本期申报抵扣
/// 功能：从导入进项发票明细清单取数
/// 备注：取值表格：进项明细清单
///       条件字段：
///       认证期间："0"当期，"-99"前期（当月之前所有月份，不含当月），"99"所有认证期间
///       发票类型：“1”增值税专票，“2”货运发票，“3”机动车发票，“4”农产品发票，“5”其他发票（包括空值）；
///       认证结果（认证通过，认证未通过，未认证）
///       取值字段：金额，税额，份数
///       在导入进项发票时会去检查是否有重复导入发票现象，不允许重复导入，从而杜绝重复抵扣。
/// </summary>
/// <param name="certificationPeriod">认证期间：“0”当期（当月），“-99”前期（当月之前所有月份，不含当月），"99"所有认证期间</param>
/// <param name="invoiceType">“1”增值税专票，“2”货运发票，“3”机动车发票，“4”农产品发票，“5”其他发票（包括空值）</param>
/// <param name="authenticationType">认证结果：“1”认证通过；“0”认证未通过；“2”未认证</param>
/// <param name="resultType">计算取值：“0”金额；“1”税额；“2”份数；</param>
/// <param name="period">会计期间——“0”当期，1-12为自然月</param>
/// <returns></returns>
public class JXFP extends FunctionBase implements FreeRefFunction {
    public JXFP(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 5) {
            return null;
        }

        int certificationPeriod = getIntParam(args[0], ec);
        int invoiceTypeParam = getIntParam(args[1], ec);
        int authenticationType = getIntParam(args[2], ec);
        int resultType = getIntParam(args[3], ec);
        int period = getIntParam(args[4], ec);
        // 客户情况只计算增值税专票
        String invoiceType = "004";

        String formulaExpression = "JXFP(" + certificationPeriod + "," + invoiceTypeParam + ","
                + authenticationType + "," + resultType + "," + period + ")";
        logger.debug(formulaExpression);

        Integer yearOffset = 0;
        certificationPeriod = getPeriod(certificationPeriod, yearOffset);

        if (yearOffset < 0) {
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal("0.0"), 0L, formulaContext.getProjectId());
            return NumberEval.ZERO;
        }

        period = getPeriod(period, yearOffset);
        if (yearOffset < 0) {
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal("0.0"), 0L, formulaContext.getProjectId());
            return NumberEval.ZERO;
        }

        Calendar calendar = Calendar.getInstance();
        calendar.set(formulaContext.getYear(), 1, 1);
        Date startDate = calendar.getTime();
        calendar.add(Calendar.YEAR, 1);
        Date endDate = calendar.getTime();

        List<InputInvoice> inputInvoices;
        if (authenticationType == 1 && formulaContext.getIsYear()) {
            inputInvoices = SpringContextUtil.inputInvoiceDao.getInputInvoice(null, invoiceType,
                    Constant.InputInvoiceCertificationResult.CheckPass, Constant.InputInvoiceCertificationResult.ScanPass,
                    null);
        } else if (authenticationType == 1) {
            inputInvoices = SpringContextUtil.inputInvoiceDao.getInputInvoice(period, invoiceType,
                    Constant.InputInvoiceCertificationResult.CheckPass, Constant.InputInvoiceCertificationResult.ScanPass,
                    null);
        }
        // 认证未通过与未认证暂认为是同一个意思
        else if ((authenticationType == 1 || authenticationType == 2) && formulaContext.getIsYear()) {
            inputInvoices = SpringContextUtil.inputInvoiceDao.getInputInvoice(null, invoiceType,
                    null, null, Constant.InputInvoiceCertificationResult.NotPass);
        }
        // 认证未通过与未认证暂认为是同一个意思
        else if (authenticationType == 0 || authenticationType == 2) {
            inputInvoices = SpringContextUtil.inputInvoiceDao.getInputInvoice(period, invoiceType, null,
                    null, Constant.InputInvoiceCertificationResult.NotPass);
        } else {
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal("0.0"), 0L, formulaContext.getProjectId());
            return NumberEval.ZERO;
        }

        if (certificationPeriod != -99 && certificationPeriod != 99
                && (certificationPeriod < 1 || certificationPeriod > 12)) {
            saveFormulaBlock(period, ec, formulaExpression, new BigDecimal("0.0"), 0L, formulaContext.getProjectId());
            return NumberEval.ZERO;
        } else if (!formulaContext.getIsYear()) {
            if (certificationPeriod == -99) {
                calendar.set(formulaContext.getYear(), formulaContext.getPeriod(), 1);
                endDate = calendar.getTime();
                Date finalEndDate = endDate;
                inputInvoices = inputInvoices.stream()
                        .filter(a -> DateUtils.strToDate(a.getRZSJ()).before(finalEndDate))
                        .collect(Collectors.toList());
            } else if (certificationPeriod != 99) {
                calendar.set(formulaContext.getYear(), certificationPeriod - 1, 1);
                startDate = calendar.getTime();
                calendar.set(formulaContext.getYear(), certificationPeriod, 1);
//                calendar.add(Calendar.MONTH, 1);
                endDate = calendar.getTime();
                Date finalEndDate1 = endDate;
                Date finalStartDate = startDate;
                inputInvoices = inputInvoices.stream()
                        .filter(a -> strToDate(a.getRZSQ()).after(finalStartDate)
                                && strToDate(a.getRZSQ()).before(finalEndDate1))
                        .collect(Collectors.toList());
            }
        }

        List<InputInvoiceDataSourceDto> dataSource = new ArrayList<>();
        for (InputInvoice x : inputInvoices) {
            InputInvoiceDataSourceDto inputInvoiceDataSourceDto = new InputInvoiceDataSourceDto();
            inputInvoiceDataSourceDto.setAmount(FormulaHelper.roundValue(new BigDecimal(x.getHJJE()), KeyValueConfigResultType.Accounting,
                    null, formulaContext));
            inputInvoiceDataSourceDto.setResultType(KeyValueConfigResultType.Accounting.getCode());
            inputInvoiceDataSourceDto.setTaxAmount(FormulaHelper.roundValue(new BigDecimal(x.getHJSE()), KeyValueConfigResultType.Accounting,
                    null, formulaContext));
            inputInvoiceDataSourceDto.setCertificationDate(DateUtils.strToDate(x.getRZSJ()));
            inputInvoiceDataSourceDto.setInvoiceCode(x.getFPDM());
            inputInvoiceDataSourceDto.setInvoiceNumber(x.getFPHM());
            inputInvoiceDataSourceDto.setInvoiceType(Integer.parseInt(x.getFPLX()));
            inputInvoiceDataSourceDto.setPeriod(period);
            inputInvoiceDataSourceDto.setSellerTaxNumber(x.getXFSH());
            inputInvoiceDataSourceDto.setName(Constant.DataSourceName.InputDetailInvoiceDataSource);
            inputInvoiceDataSourceDto.setOperationType(EnumOperationType.Single.getCode());
            dataSource.add(inputInvoiceDataSourceDto);
        }

        dataSource = dataSource.stream()
                .sorted(Comparator.comparing(InputInvoiceDataSourceDto::getCertificationDate)
                        .thenComparing(InputInvoiceDataSourceDto::getInvoiceCode)
                        .thenComparing(InputInvoiceDataSourceDto::getInvoiceNumber))
                .collect(Collectors.toList());

        if (resultType == 0) {
            double val = dataSource.stream().mapToDouble(a -> a.getAmount().doubleValue()).sum();
            Long dataSourceId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.InputInvoiceDataSourceDto,
                    BigDecimal.valueOf(val), formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(),
                    formulaContext.getProjectId());
            saveFormulaBlock(formulaContext.getPeriod(), ec, formulaExpression, BigDecimal.valueOf(val), dataSourceId,
                    formulaContext.getProjectId());
            return new NumberEval(val);
        } else if (resultType == 1) {
            double val = dataSource.stream().mapToDouble(a -> a.getTaxAmount().doubleValue()).sum();
            Long dataSourceId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.InputInvoiceDataSourceDto,
                    BigDecimal.valueOf(val), formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(formulaContext.getPeriod(), ec, formulaExpression, BigDecimal.valueOf(val), dataSourceId,
                    formulaContext.getProjectId());
            return new NumberEval(val);
        } else if (resultType == 2) {
            double val = dataSource.size();
            Long dataSourceId = saveDataSource(ec, Collections.singletonList(dataSource),
                    FormulaDataSourceDetailType.InputInvoiceDataSourceDto,
                    BigDecimal.valueOf(val), formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
            saveFormulaBlock(period, ec, formulaExpression, BigDecimal.valueOf(val), dataSourceId,
                    formulaContext.getProjectId());
            return new NumberEval(val);
        }
        saveFormulaBlock(period, ec, formulaExpression, new BigDecimal("0.0"), 0L,
                formulaContext.getProjectId());
        return NumberEval.ZERO;
    }

    private static Date strToDate(String strDate) {
        strDate = strDate + 15;
        SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMdd");
        ParsePosition pos = new ParsePosition(0);
        Date strtodate = formatter.parse(strDate, pos);
        return strtodate;
    }

}
