package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.vat.dao.HlOutputInterfaceLineMapper;
import pwc.taxtech.atms.vat.dao.HlOutputOrderKpsqmxbMapper;
import pwc.taxtech.atms.vat.dao.HlOutputOrderKpsqzbMapper;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

public class WPSR extends FunctionBase implements FreeRefFunction {
    private static final ValueEval ZERO_EVAL = new StringEval("0");
    private HlOutputInterfaceLineMapper interfaceLineMapper;
    private HlOutputOrderKpsqmxbMapper kpsqmxbMapper;
    private HlOutputOrderKpsqzbMapper zbMapper;

    public WPSR(FormulaContext formulaContext) {
        super(formulaContext);
        this.interfaceLineMapper = SpringContextUtil.getBean(HlOutputInterfaceLineMapper.class);
        this.kpsqmxbMapper = SpringContextUtil.getBean(HlOutputOrderKpsqmxbMapper.class);
        this.zbMapper = SpringContextUtil.getBean(HlOutputOrderKpsqzbMapper.class);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length != 2) {
            return ZERO_EVAL;
        }
        Integer type = getIntParam(args[0], ec);
        Integer rate = getIntParam(args[1], ec);
        String rateStr = String.valueOf(rate / 100);
        if (null == type || null == rate) {
            return ZERO_EVAL;
        }
        ValueEval result = ZERO_EVAL;
        try {
            int period = formulaContext.getPeriod();
            int year = formulaContext.getYear();
            Project project = SpringContextUtil.projectMapper.selectByPrimaryKey(formulaContext.getProjectId());
            Organization organization = SpringContextUtil.organizationMapper.selectByPrimaryKey(project.getOrganizationId());
            String periodStart = DateUtils.getPeriodBeginFormat(project.getYear(), period);
            String periodEnd = DateUtils.getPeriodEndFormat(project.getYear(), period);

            if (1 == type) {
                //未开票收入
                HlOutputInterfaceLineExample lineExample = new HlOutputInterfaceLineExample();
                lineExample.createCriteria().andSjfyrqBetween(periodStart, periodEnd).andSfkpNotEqualTo("1")
                        .andEbsSrbjEqualTo("1").andYwstidEqualTo(organization.getCode()); //排除 状态1：已开票，即未开票和预开票
                List<HlOutputInterfaceLine> lineList = interfaceLineMapper.selectByExample(lineExample); //todo 无订单业务
                if (!CollectionUtils.isEmpty(lineList)) {
                    result = new StringEval(getSum(lineList, rateStr).toPlainString());
                }
            } else if (2 == type) {
                //跨月开票
                HlOutputInterfaceLineExample lineExample = new HlOutputInterfaceLineExample();
                lineExample.createCriteria().andSjfyrqNotBetween(periodStart, periodEnd).andKprqBetween(periodStart, periodEnd)
                        .andSfkpEqualTo("1").andEbsSrbjEqualTo("1").andYwstidEqualTo(organization.getCode());
                List<HlOutputInterfaceLine> lineList = interfaceLineMapper.selectByExample(lineExample);
                if (!CollectionUtils.isEmpty(lineList)) {
                    result = new StringEval(getSum(lineList, rateStr).toPlainString());
                }
            }
        } catch (Exception e) {
            logger.error("evaluate WPSR error.", e);
        }
        return result;
    }

    /**
     * 接口行数据 -> 发票明细  求不含税金额总和
     *
     * @param lineList List<HlOutputInterfaceLine>
     * @param rateStr  eg：0.16
     * @return BigDecimal
     */
    private BigDecimal getSum(List<HlOutputInterfaceLine> lineList, String rateStr) {
        List<String> idList = lineList.stream().map(HlOutputInterfaceLine::getUuid).collect(Collectors.toList());
        HlOutputOrderKpsqmxbExample mxbExample = new HlOutputOrderKpsqmxbExample();
        mxbExample.createCriteria().andIdIn(idList);
        List<String> sqList = kpsqmxbMapper.selectByExample(mxbExample).stream().map(HlOutputOrderKpsqmxb::getSquuid)
                .collect(Collectors.toList());
        HlOutputOrderKpsqzbExample zbExample = new HlOutputOrderKpsqzbExample();
        zbExample.createCriteria().andSquuidIn(sqList);
        List<String> sqbhList = zbMapper.selectByExample(zbExample).stream().map(HlOutputOrderKpsqzb::getSqbh)
                .collect(Collectors.toList());
        OutputInvoiceExample outputInvoiceExample = new OutputInvoiceExample();
        outputInvoiceExample.createCriteria().andSQBHIn(sqbhList);
        List<String> lshList = SpringContextUtil.outputInvoiceMapper.selectByExample(outputInvoiceExample).stream()
                .map(OutputInvoice::getFPQQLSH).collect(Collectors.toList());
        OutputInvoiceDetailExample detailExample = new OutputInvoiceDetailExample();
        detailExample.createCriteria().andFPQQLSHIn(lshList).andSLVEqualTo(rateStr);
        return SpringContextUtil.outputInvoiceDetailMapper.selectByExample(detailExample).stream()
                .map(o -> NumberUtils.createBigDecimal(o.getJE())).reduce(BigDecimal.ZERO, BigDecimal::add);
    }
}
